/*
    Copyright (C) 2005-2010  Erik van Pienbroek

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <glib.h>
#include <glib-object.h>
#include "nntpgrab_utils.h"

#ifdef HAVE_SOUP
#include <libsoup/soup.h>
#endif
#ifdef HAVE_SOUP_GNOME
#include <libsoup/soup-gnome.h>
#endif

NGTypeInstance*   ng_type_check_instance_cast   (NGTypeInstance      *instance,
                                                 NGType               iface_type)
{
    return (NGTypeInstance*) g_type_check_instance_cast((GTypeInstance*) instance, (GType) iface_type);
}

void nntpgrab_utils_perform_base_initialization(void)
{
    if (!g_thread_supported ()) g_thread_init (NULL);
    g_type_init();

#ifdef HAVE_SOUP
    /* Due to a bug in libsoup we need to make sure that all GObject's
     * are initialized from the main thread to avoid a deadlock when libsoup
     * is called for the first time from a different thread */
    SoupSession *session = soup_session_sync_new();

#if 0
    SoupLogger *logger = soup_logger_new(SOUP_LOGGER_LOG_BODY, -1);
    soup_session_add_feature(session, SOUP_SESSION_FEATURE(logger));
    g_object_unref(logger);
#endif

#ifdef HAVE_SOUP_GNOME
    /* Enable proxy support when using GNOME 2.26.0 or higher */
    soup_session_add_feature_by_type(session, SOUP_TYPE_GNOME_FEATURES_2_26);
#endif

#ifdef HAVE_SOUP_GZIP
    /* Enable gzip compression */
    soup_session_add_feature_by_type(session, SOUP_TYPE_CONTENT_DECODER);
#endif

    g_object_unref(session);
#endif
}
