/*
    Copyright (C) 2005-2010  Erik van Pienbroek

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "gui.h"

G_MODULE_EXPORT void
on_chkEnableWebserver_toggled(GtkToggleButton *caller, gpointer data)
{
    NGConfigOpts opts = nntpgrab_glue_config_get_opts(glue);
    gboolean state = gtk_toggle_button_get_active(caller);

    if (opts.enable_webserver != state) {
        opts.enable_webserver = state;
    }

    nntpgrab_glue_config_set_opts(glue, opts);
}

G_MODULE_EXPORT void
on_btnConfigWebserverPortApply_clicked(GtkButton *caller, gpointer data)
{
    NGConfigOpts opts = nntpgrab_glue_config_get_opts(glue);
    GtkWidget *spinConfigWebserverPort = nntpgrab_gui_base_get_widget("spinConfigWebserverPort");
    int new_value = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinConfigWebserverPort));

    if (new_value != opts.webserver_port) {
        opts.webserver_port = new_value;
        nntpgrab_glue_config_set_opts(glue, opts);
    }
}

G_MODULE_EXPORT void
on_spinConfigWebserverPort_value_changed(GtkSpinButton *caller, gpointer data)
{
    GtkWidget *btnConfigWebserverPortApply = nntpgrab_gui_base_get_widget("btnConfigWebserverPortApply");

    gtk_widget_set_sensitive(btnConfigWebserverPortApply, TRUE);
}

G_MODULE_EXPORT void
on_chkEnableLogger_toggled(GtkToggleButton *caller, gpointer data)
{
    NGConfigOpts opts = nntpgrab_glue_config_get_opts(glue);
    gboolean state = gtk_toggle_button_get_active(caller);

    if (opts.enable_logger != state) {
        opts.enable_logger = state;
    }

    nntpgrab_glue_config_set_opts(glue, opts);
}

static void
config_changed(NntpgrabGlue *obj, gpointer data)
{
    GtkWidget *chkEnableWebserver = nntpgrab_gui_base_get_widget("chkEnableWebserver");
    GtkWidget *spinConfigWebserverPort = nntpgrab_gui_base_get_widget("spinConfigWebserverPort");
    GtkWidget *btnConfigWebserverPortApply = nntpgrab_gui_base_get_widget("btnConfigWebserverPortApply");
    GtkWidget *labelConfigWebserverPort = nntpgrab_gui_base_get_widget("labelConfigWebserverPort");
    GtkWidget *chkEnableLogger = nntpgrab_gui_base_get_widget("chkEnableLogger");
    NGConfigOpts opts = nntpgrab_glue_config_get_opts(glue);

    g_signal_handlers_block_by_func (G_OBJECT (chkEnableWebserver), G_CALLBACK (on_chkEnableWebserver_toggled), NULL);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chkEnableWebserver), opts.enable_webserver);
    g_signal_handlers_unblock_by_func (G_OBJECT (chkEnableWebserver), G_CALLBACK (on_chkEnableWebserver_toggled), NULL);

    g_signal_handlers_block_by_func (G_OBJECT (chkEnableLogger), G_CALLBACK (on_chkEnableLogger_toggled), NULL);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chkEnableLogger), opts.enable_logger);
    g_signal_handlers_unblock_by_func (G_OBJECT (chkEnableLogger), G_CALLBACK (on_chkEnableLogger_toggled), NULL);

    if (!opts.enable_webserver) {
        gtk_widget_hide(spinConfigWebserverPort);
        gtk_widget_hide(labelConfigWebserverPort);
        gtk_widget_hide(btnConfigWebserverPortApply);
    } else {
        gtk_widget_show(spinConfigWebserverPort);
        gtk_widget_show(labelConfigWebserverPort);
        gtk_widget_show(btnConfigWebserverPortApply);

        gtk_widget_set_sensitive(btnConfigWebserverPortApply, FALSE);
        gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinConfigWebserverPort), opts.webserver_port);
    }
}

static void
on_plugin_loaded (NntpgrabGlue *obj, const char *plugin_name, gboolean is_persistent)
{
    GtkWidget *framePluginWebserver = nntpgrab_gui_base_get_widget("framePluginWebserver");
    GtkWidget *framePluginLogger = nntpgrab_gui_base_get_widget("framePluginLogger");

    if (!strcmp(plugin_name, "JSON-RPC")) {
        gtk_widget_set_sensitive(framePluginWebserver, TRUE);
    } else if (!strcmp(plugin_name, "Logger")) {
        gtk_widget_set_sensitive(framePluginLogger, TRUE);
    }
}

static void
on_plugin_unloaded (NntpgrabGlue *obj, const char *plugin_name)
{
    GtkWidget *framePluginWebserver = nntpgrab_gui_base_get_widget("framePluginWebserver");
    GtkWidget *framePluginLogger = nntpgrab_gui_base_get_widget("framePluginLogger");

    if (!strcmp(plugin_name, "JSON-RPC")) {
        gtk_widget_set_sensitive(framePluginWebserver, FALSE);
    } else if (!strcmp(plugin_name, "Logger")) {
        gtk_widget_set_sensitive(framePluginLogger, FALSE);
    }
}

void
config_plugins_initialize()
{
    GtkWidget *framePluginWebserver = nntpgrab_gui_base_get_widget("framePluginWebserver");
    GtkWidget *framePluginLogger = nntpgrab_gui_base_get_widget("framePluginLogger");
    NGList *list;
    NGList *avail_plugins;

    nntpgrab_glue_signal_connect(glue, "config_changed", NG_CALLBACK(config_changed), NULL);
    config_changed(glue, NULL);

    nntpgrab_glue_signal_connect(glue, "plugin_loaded", NG_CALLBACK(on_plugin_loaded), NULL);
    nntpgrab_glue_signal_connect(glue, "plugin_unloaded", NG_CALLBACK(on_plugin_unloaded), NULL);

    gtk_widget_set_sensitive(framePluginWebserver, FALSE);
    gtk_widget_set_sensitive(framePluginLogger, FALSE);
    avail_plugins = nntpgrab_glue_plugins_get_avail_plugins(glue);
    list = avail_plugins;
    while (list) {
        const char *plugin_name = list->data;
        NNTPGrabPluginInfo plugin_info;

        if (!nntpgrab_glue_plugins_get_plugin_info(glue, plugin_name, &plugin_info)) {
            /* Ignore */
            list = ng_list_next(list);
            continue;
        }

        if (!plugin_info.is_loaded) {
            list = ng_list_next(list);
            continue;
        }

        on_plugin_loaded(glue, plugin_name, FALSE);

        list = ng_list_next(list);
    }

    nntpgrab_glue_plugins_free_avail_plugins(glue, avail_plugins);
}
