#include "DialogAddServer.h"
#include "ui_DialogAddServer.h"

DialogAddServer::DialogAddServer(QNNTPGrabGlue *glue, QNNTPGrabGlue::ConfigServer *server, QWidget *parent) :
        QDialog(parent),
        ui(new Ui::DialogAddServer)
{
    ui->setupUi(this);
    this->glue = glue;
    this->server = server;

    if(this->server != NULL) {
        ui->txt_name->setText(server->servername);
        ui->txt_address->setText(server->hostname);

        if(server->username.length() > 0) {
            ui->cb_requires_authentication->setChecked(true);
            ui->txt_username->setText(server->username);
            ui->txt_password->setText(server->password);
        }
        ui->sp_port->setValue(server->port);
        ui->sp_connections->setValue(server->max_threads);
        ui->cmb_priority->setCurrentIndex(server->priority);

        ui->cb_ssl->setChecked(server->use_ssl);
        ui->cb_enabled->setChecked(server->enabled);
    }
}

DialogAddServer::~DialogAddServer()
{
    delete ui;
}

void DialogAddServer::on_cb_requires_authentication_toggled(bool checked)
{
    ui->txt_username->setEnabled(checked);
    ui->txt_password->setEnabled(checked);
    //empty text?
}

void DialogAddServer::changeEvent(QEvent *e)
{
    QDialog::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void DialogAddServer::on_btn_apply_clicked()
{
    QString errmsg = "";
    if (ui->txt_name->text().isEmpty()) {
        qWarning("%s", tr("Please enter a servername").toUtf8().data());
        return;
    }

    if (ui->txt_address->text().isEmpty()) {
        qWarning("%s", tr("Please enter a hostname").toUtf8().data());
        return;
    }

    QNNTPGrabGlue::ConfigServer server;

    server.servername = ui->txt_name->text();
    server.hostname = ui->txt_address->text();

    if (ui->cb_requires_authentication->isChecked()) {
        server.username = ui->txt_username->text();
        server.password = ui->txt_password->text();
    } else {
        server.username = QString();
        server.password = QString();
    }

    server.port = ui->sp_port->value();
    server.max_threads = ui->sp_connections->value();
    server.priority = (QNNTPGrabGlue::ServerPriority) ui->cmb_priority->currentIndex();
    server.use_ssl = ui->cb_ssl->isChecked();
    server.enabled = ui->cb_enabled->isChecked();

    if (this->server != NULL) {
        if (!glue->configEditServer(QString(this->server->servername), server, errmsg)) {
            qWarning("%s", errmsg.toUtf8().data());
            return;
        }
    } else {
        if (!glue->configAddServer(server, errmsg)) {
            qWarning("%s", errmsg.toUtf8().data());
            return;
        }
    }

    this->close();
}

void DialogAddServer::on_btn_Cancel_clicked()
{
    this->close();
}

void DialogAddServer::on_cb_ssl_toggled(bool checked)
{
    if (checked && ui->sp_port->value() == 119) {
        ui->sp_port->setValue(563);
    } else if(!checked && ui->sp_port->value() == 563) {
        ui->sp_port->setValue(119);
    }
}
