#include "DialogConnect.h"
#include "ui_DialogConnect.h"

#include <QMessageBox>
#include "ConfigGuiOpts.h"
#include "QNNTPGrabGlue.h"
#include "mainwindow.h"

DialogConnect::DialogConnect(NGApplication *app, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::DialogConnect)
{
    glue = new QNNTPGrabGlue();

    this->app = app;

    ui->setupUi(this);
    ui->btnStartStandalone->setFocus();
    w = NULL;

    if (glue->utilsTestIsServerAlreadyRunning()) {
        ui->btnStartStandalone->setEnabled(false);
    }
}

DialogConnect::~DialogConnect()
{
    if (w) {
        delete w;
    }

    delete ui;
}

void DialogConnect::changeEvent(QEvent *e)
{
    QDialog::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void DialogConnect::doStartStandalone(void)
{
    doConnect(NULL, 0);
}

void DialogConnect::doConnect(QString hostname, int port)
{
    QString errmsg = "";
    QString warnings = "";

    if (!glue->init(hostname, port, NULL, NULL, false, errmsg, warnings)) {
        QMessageBox b(this);

        b.setWindowTitle("NNTPGrab");
        b.setText(QString(tr("Unable to connect to an NNTPGrab server:\n")) + errmsg);
        b.show();
        b.exec();

        return;
    }

    if (!warnings.isEmpty()) {
        qWarning("%s", warnings.toUtf8().data());
    }

    w = new MainWindow(glue, app);
    w->show();

    if (ui->chkDontShowThisDialogAgain->isChecked()) {
        gui_opts.setShowConnectDialog(FALSE);
    }

    this->close();
}

void DialogConnect::on_btnStartStandalone_clicked()
{
    doConnect(NULL, 0);
}

void DialogConnect::on_btnConnect_clicked()
{
    doConnect(ui->editHostname->text(), ui->spinPort->value());
}
