#include <QMetaType>

#include <stdlib.h>
#include <string.h>
#ifdef WIN32
#include <windows.h>
#endif

#include "QNNTPGrabGlue.h"
#include "QNNTPGrabGlue_p.h"
#include "nntpgrab_glue.h"
#include "nntpgrab_utils.h"

QNNTPGrabGlueThread::QNNTPGrabGlueThread(NntpgrabGlue *glue)
{
    this->glue = glue;
}

QNNTPGrabGlueThread::~QNNTPGrabGlueThread()
{
    if (this->isRunning()) {
        ng_event_handler_loop_quit();
        this->wait();
    }
}

void QNNTPGrabGlueThread::run()
{
    ng_event_handler_loop_run();
}

QNNTPGrabGluePrivate::QNNTPGrabGluePrivate(QNNTPGrabGlue *parent) : q_ptr(parent)
{
}

QNNTPGrabGluePrivate::~QNNTPGrabGluePrivate()
{
}

QNNTPGrabGlue::QNNTPGrabGlue() : d_ptr(new QNNTPGrabGluePrivate(this))
{
    Q_D(QNNTPGrabGlue);
    d->q_ptr = this;
    d->glue = NULL;

    qRegisterMetaType<QNNTPGrabGlue::NNTPDisconnectType>("QNNTPGrabGlue::NNTPDisconnectType");
    qRegisterMetaType<QNNTPGrabGlue::TaskState>("QNNTPGrabGlue::TaskState");
    qRegisterMetaType<QNNTPGrabGlue::LogLevel>("QNNTPGrabGlue::LogLevel");
    qRegisterMetaType<QNNTPGrabGlue::SchedularState>("QNNTPGrabGlue::SchedularState");
    qRegisterMetaType<QList<QString> >("QList<QString>");
}

QNNTPGrabGlue::~QNNTPGrabGlue()
{
    Q_D(QNNTPGrabGlue);
    if (d->glue) {
        nntpgrab_glue_destroy(d->glue);
        d->glue = NULL;
    }
}

/*******************************************
 * Callback wrappers                       *
 *******************************************/
void QNNTPGrabGluePrivate::connectionLostWrapper(NntpgrabGlue *glue, const char *errmsg, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->connectionLost(QString(errmsg));
}

void QNNTPGrabGluePrivate::configChangedWrapper(NntpgrabGlue *glue, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->configChanged();
}

void QNNTPGrabGluePrivate::allDownloadsCompletedWrapper(NntpgrabGlue *glue, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->allDownloadsCompleted();
}

void QNNTPGrabGluePrivate::partDownloadStartWrapper(NntpgrabGlue *glue, const char *servername, int conn_id, const char *collection_name, const char *subject, int part_num, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->partDownloadStart(QString(servername), conn_id, QString(collection_name), QString(subject), part_num);
}

void QNNTPGrabGluePrivate::partDoneWrapper(NntpgrabGlue *glue, const char *servername, int conn_id, const char *collection_name, const char *subject, int part_num, int size, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->partDone(QString(servername), conn_id, QString(collection_name), QString(subject), part_num, size);
}

void QNNTPGrabGluePrivate::partFailedWrapper(NntpgrabGlue *glue, const char *servername, int conn_id, const char *collection_name, const char *subject, int part_num,int size, bool all_servers_tried, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->partFailed(QString(servername), conn_id, QString(collection_name), QString(subject), part_num, size, all_servers_tried);
}

void QNNTPGrabGluePrivate::partProgressUpdateWrapper(NntpgrabGlue *glue, const char *servername, int conn_id, const char *collection_name, const char *subject, int part_num, int bytes_downloaded, int bytes_total, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->partProgressUpdate(QString(servername), conn_id, QString(collection_name), QString(subject), part_num, bytes_downloaded, bytes_total);
}

void QNNTPGrabGluePrivate::trafficMonitorUpdateWrapper(NntpgrabGlue *glue, int bytes_received1, int bytes_received2, int bytes_received3, int bytes_received4, int bytes_received5, int bytes_received6, int bytes_received7, int bytes_received8, int bytes_received9, int bytes_received10, nguint64 stamp, double average, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->trafficMonitorUpdate(bytes_received1, bytes_received2, bytes_received3, bytes_received4, bytes_received5, bytes_received6,
                                                       bytes_received7, bytes_received8, bytes_received9, bytes_received10, QDateTime::fromTime_t(stamp), average);
}

void QNNTPGrabGluePrivate::collectionAddedWrapper(NntpgrabGlue *glue, const char *collection_name, const char *poster, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->collectionAdded(QString(collection_name), QString(poster));
}

void QNNTPGrabGluePrivate::collectionModifiedWrapper(NntpgrabGlue *glue, const char *collection_name, const char *poster, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->collectionModified(QString(collection_name), QString(poster));
}

void QNNTPGrabGluePrivate::collectionRemovedWrapper(NntpgrabGlue *glue, const char *collection_name, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->collectionRemoved(QString(collection_name));
}

void QNNTPGrabGluePrivate::fileAddedWrapper(NntpgrabGlue *glue, const char *collection_name, const char *subject, const char *poster, nguint64 stamp, nguint64 file_size, nguint64 total_size, nguint64 total_size_remaining, NGTaskState state, int num_parts, NGList *groups, void *data)
{
    Q_UNUSED(glue);

    QList<QString> list_groups;
    NGList *list;

    list = groups;
    while (list) {
        list_groups.append(QString((const char *) list->data));
        list = ng_list_next(list);
    }

    emit ((QNNTPGrabGlue*) data)->fileAdded(QString(collection_name), QString(subject), QString(poster), QDateTime::fromTime_t(stamp), file_size, total_size, total_size_remaining, (QNNTPGrabGlue::TaskState) state, num_parts, list_groups);
}

void QNNTPGrabGluePrivate::fileRemovedWrapper(NntpgrabGlue *glue, const char *collection_name, const char *subject, nguint64 total_size, nguint64 total_size_remaining, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->fileRemoved(QString(collection_name), QString(subject), total_size, total_size_remaining);
}

void QNNTPGrabGluePrivate::fileDownloadStateUpdateWrapper(NntpgrabGlue *glue, const char *collection_name, const char *subject, int num_parts_total, int num_parts_done, int num_parts_failed, nguint64 file_size, nguint64 file_size_remaining, nguint64 total_size, nguint64 total_size_remaining, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->fileDownloadStateUpdate(QString(collection_name), QString(subject), num_parts_total, num_parts_done, num_parts_failed, file_size, file_size_remaining, total_size, total_size_remaining);
}

void QNNTPGrabGluePrivate::fileStateChangedWrapper(NntpgrabGlue *glue, const char *collection_name, const char *subject, const char *real_filename, NGTaskState old_state, NGTaskState new_state, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->fileStateChanged(QString(collection_name), QString(subject), QString(real_filename), (QNNTPGrabGlue::TaskState) old_state, (QNNTPGrabGlue::TaskState) new_state);
}

void QNNTPGrabGluePrivate::connectionConnectingWrapper(NntpgrabGlue *glue, const char *servername, int conn_id, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->connectionConnecting(QString(servername), conn_id);
}

void QNNTPGrabGluePrivate::connectionConnectedWrapper(NntpgrabGlue *glue, const char *servername, int conn_id, const char *welcome_msg, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->connectionConnected(QString(servername), conn_id, welcome_msg);
}

void QNNTPGrabGluePrivate::connectionDisconnectWrapper(NntpgrabGlue *glue, const char *servername, int conn_id, NNTPDisconnectType disconnect_type, const char *reason, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->connectionDisconnect(QString(servername), conn_id, (QNNTPGrabGlue::NNTPDisconnectType) disconnect_type, QString(reason));
}

void QNNTPGrabGluePrivate::logMessageWrapper(NntpgrabGlue *glue, const char *component, NGLogLevel log_level, const char *msg, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->logMessage(QString(component), (QNNTPGrabGlue::LogLevel) log_level, QString(msg));
}

void QNNTPGrabGluePrivate::schedularStateChangedWrapper(NntpgrabGlue *glue, NGSchedularState new_state, const char *reason, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->schedularStateChanged((QNNTPGrabGlue::SchedularState) new_state, QString(reason));
}

void QNNTPGrabGluePrivate::fileMovedWrapper(NntpgrabGlue *glue, const char *orig_collection_name, const char *subject, const char *new_collection_name, int old_position, int new_position, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->fileMoved(QString(orig_collection_name), QString(subject), QString(new_collection_name), old_position, new_position);
}

void QNNTPGrabGluePrivate::collectionMovedWrapper(NntpgrabGlue *glue, const char *collection_name, int old_position, int new_position, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->collectionMoved(QString(collection_name), old_position, new_position);
}

void QNNTPGrabGluePrivate::collectionDownloadedWrapper(NntpgrabGlue *glue, const char *collection_name, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->collectionDownloaded(QString(collection_name));
}

void QNNTPGrabGluePrivate::pluginLoadedWrapper(NntpgrabGlue *glue, const char *plugin_name, bool is_persistent, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->pluginLoaded(QString(plugin_name), is_persistent);
}

void QNNTPGrabGluePrivate::pluginUnloadedWrapper(NntpgrabGlue *glue, const char *plugin_name, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->pluginUnloaded(QString(plugin_name));
}

void QNNTPGrabGluePrivate::pluginEventWrapper(NntpgrabGlue *glue, const char *plugin_name, const char *event_name, const char **values, void *data)
{
    Q_UNUSED(glue);

    int i;
    QList<QString> list_values;

    i = 0;
    while (values[i]) {
        list_values.append(QString(values[i]));
        i++;
    }

    emit ((QNNTPGrabGlue*) data)->pluginEvent(QString(plugin_name), QString(event_name), list_values);
}

/*************************
 * Public methods        *
 *************************/
bool QNNTPGrabGlue::init(QString hostname, int port, QString username, QString password, bool use_ssl, QString &errmsg, QString &warnings)
{
    Q_D(QNNTPGrabGlue);

    char *errmsg_tmp = NULL;
    char *warnings_tmp = NULL;

    nntpgrab_utils_perform_base_initialization();

    d->glue = nntpgrab_glue_new();

    if (!nntpgrab_glue_init(d->glue, NNTPGRAB_GLUE_VERSION, &errmsg_tmp)) {
        errmsg.clear();
        errmsg.append(errmsg_tmp);
        ng_free(errmsg_tmp);

        nntpgrab_glue_destroy(d->glue);
        d->glue = NULL;
        return false;
    }

    if (!nntpgrab_glue_connect(d->glue, hostname.toUtf8(), port, username.toUtf8(), password.toUtf8(), use_ssl, &errmsg_tmp, &warnings_tmp)) {
        errmsg.clear();
        errmsg.append(errmsg_tmp);
        ng_free(errmsg_tmp);

        if (warnings_tmp) {
            warnings.clear();
            warnings.append(warnings_tmp);
            ng_free(warnings_tmp);
        }

        return false;
    }

    if (warnings_tmp) {
        warnings.clear();
        warnings.append(warnings_tmp);
        ng_free(warnings_tmp);
    }

    if (!hostname.isEmpty()) {
        nntpgrab_glue_signal_connect(d->glue, "connection_lost", NG_CALLBACK(d->connectionLostWrapper), this);
    }

    nntpgrab_glue_signal_connect(d->glue, "config_changed", NG_CALLBACK(d->configChangedWrapper), this);
    nntpgrab_glue_signal_connect(d->glue, "all_downloads_completed", NG_CALLBACK(d->allDownloadsCompletedWrapper), this);
    nntpgrab_glue_signal_connect(d->glue, "part_download_start", NG_CALLBACK(d->partDownloadStartWrapper), this);
    nntpgrab_glue_signal_connect(d->glue, "part_done", NG_CALLBACK(d->partDoneWrapper), this);
    nntpgrab_glue_signal_connect(d->glue, "part_failed", NG_CALLBACK(d->partFailedWrapper), this);
    nntpgrab_glue_signal_connect(d->glue, "traffic_monitor_update", NG_CALLBACK(d->trafficMonitorUpdateWrapper), this);
    nntpgrab_glue_signal_connect(d->glue, "part_progress_update", NG_CALLBACK(d->partProgressUpdateWrapper), this);
    nntpgrab_glue_signal_connect(d->glue, "collection_added", NG_CALLBACK(d->collectionAddedWrapper), this);
    nntpgrab_glue_signal_connect(d->glue, "collection_modified", NG_CALLBACK(d->collectionModifiedWrapper), this);
    nntpgrab_glue_signal_connect(d->glue, "collection_removed", NG_CALLBACK(d->collectionRemovedWrapper), this);
    nntpgrab_glue_signal_connect(d->glue, "file_added", NG_CALLBACK(d->fileAddedWrapper), this);
    nntpgrab_glue_signal_connect(d->glue, "file_removed", NG_CALLBACK(d->fileRemovedWrapper), this);
    nntpgrab_glue_signal_connect(d->glue, "file_download_state_update", NG_CALLBACK(d->fileDownloadStateUpdateWrapper), this);
    nntpgrab_glue_signal_connect(d->glue, "file_state_changed", NG_CALLBACK(d->fileStateChangedWrapper), this);
    nntpgrab_glue_signal_connect(d->glue, "connection_connecting", NG_CALLBACK(d->connectionConnectingWrapper), this);
    nntpgrab_glue_signal_connect(d->glue, "connection_connected", NG_CALLBACK(d->connectionConnectedWrapper), this);
    nntpgrab_glue_signal_connect(d->glue, "connection_disconnect", NG_CALLBACK(d->connectionDisconnectWrapper), this);
    nntpgrab_glue_signal_connect(d->glue, "schedular_state_changed", NG_CALLBACK(d->schedularStateChangedWrapper), this);
    nntpgrab_glue_signal_connect(d->glue, "log_message", NG_CALLBACK(d->logMessageWrapper), this);
    nntpgrab_glue_signal_connect(d->glue, "task_moved", NG_CALLBACK(d->fileMovedWrapper), this);
    nntpgrab_glue_signal_connect(d->glue, "collection_moved", NG_CALLBACK(d->collectionMovedWrapper), this);
    nntpgrab_glue_signal_connect(d->glue, "collection_downloaded", NG_CALLBACK(d->collectionDownloadedWrapper), this);
    nntpgrab_glue_signal_connect(d->glue, "plugin_loaded", NG_CALLBACK(d->pluginLoadedWrapper), this);
    nntpgrab_glue_signal_connect(d->glue, "plugin_unloaded", NG_CALLBACK(d->pluginUnloadedWrapper), this);
    nntpgrab_glue_signal_connect(d->glue, "plugin_event", NG_CALLBACK(d->pluginEventWrapper), this);

    /* If Qt is compiled with GLib support then there's already an GLib main loop active */
    /* If this isn't the case, start one in a seperate thread */
    if (ng_event_handler_loop_depth() == 0) {
        d->callbacks_thread = new QNNTPGrabGlueThread(d->glue);
        d->callbacks_thread->start();
    }

    return true;
}

bool QNNTPGrabGlue::getIsConnected()
{
    Q_D(QNNTPGrabGlue);
    return nntpgrab_glue_get_is_connected(d->glue);
}

bool QNNTPGrabGlue::getIsStandalone()
{
    Q_D(QNNTPGrabGlue);
    return nntpgrab_glue_get_is_standalone(d->glue);
}

QList<QString> QNNTPGrabGlue::configGetAvailServers()
{
    Q_D(QNNTPGrabGlue);
    QList<QString> ret;
    NGList *servers, *list;

    servers = nntpgrab_glue_config_get_avail_servers(d->glue);
    list = servers;
    while (list) {
        const char *servername = (const char *) list->data;

        ret.append(QString(servername));

        list = ng_list_next(list);
    }
    nntpgrab_glue_config_free_avail_servers(d->glue, servers);

    return ret;
}

bool QNNTPGrabGlue::configGetServerInfo(QString servername, QNNTPGrabGlue::ConfigServer &server)
{
    Q_D(QNNTPGrabGlue);
    NGConfigServer tmp;
    memset(&tmp, 0, sizeof(tmp));
    bool ret = nntpgrab_glue_config_get_server_info(d->glue, servername.toUtf8(), &tmp);

    if (ret) {
        server.servername = QString(tmp.servername);
        server.hostname = QString(tmp.hostname);
        server.port = tmp.port;
        server.username = QString(tmp.username);
        server.password = QString(tmp.password);
        server.max_threads = tmp.max_threads;
        server.priority = (QNNTPGrabGlue::ServerPriority) tmp.priority;
        server.use_ssl = tmp.use_ssl;
        server.enabled = tmp.enabled;
    }

    return ret;
}

bool QNNTPGrabGlue::configAddServer(QNNTPGrabGlue::ConfigServer new_server, QString &errmsg)
{
    Q_D(QNNTPGrabGlue);
    char *errmsg_tmp = NULL;
    NGConfigServer tmp;
    memset(&tmp, 0, sizeof(tmp));

    strncpy(tmp.servername, new_server.servername.toUtf8().data(), sizeof(tmp.servername) -1);
    strncpy(tmp.hostname, new_server.hostname.toUtf8().data(), sizeof(tmp.hostname) -1);
    tmp.port = new_server.port;
    strncpy(tmp.username, new_server.username.toUtf8().data(), sizeof(tmp.username) -1);
    strncpy(tmp.password, new_server.password.toUtf8().data(), sizeof(tmp.password) -1);
    tmp.max_threads = new_server.max_threads;
    tmp.priority = (NGServerPriority) new_server.priority;
    tmp.use_ssl = new_server.use_ssl;
    tmp.enabled = new_server.enabled;

    bool ret = nntpgrab_glue_config_add_server(d->glue, tmp, &errmsg_tmp);

    if (!ret) {
        errmsg.clear();
        errmsg.append(errmsg_tmp);
        ng_free(errmsg_tmp);
    }

    return ret;
}

bool QNNTPGrabGlue::configDelServer(QString servername, QString &errmsg)
{
    Q_D(QNNTPGrabGlue);
    char *errmsg_tmp = NULL;
    bool ret = nntpgrab_glue_config_del_server(d->glue, servername.toUtf8(), &errmsg_tmp);

    if (!ret) {
        errmsg.clear();
        errmsg.append(errmsg_tmp);
        ng_free(errmsg_tmp);
    }

    return ret;
}

bool QNNTPGrabGlue::configEditServer(QString servername, QNNTPGrabGlue::ConfigServer server, QString &errmsg)
{
    Q_D(QNNTPGrabGlue);
    char *errmsg_tmp = NULL;
    NGConfigServer tmp;
    memset(&tmp, 0, sizeof(tmp));

    strncpy(tmp.servername, server.servername.toUtf8().data(), sizeof(tmp.servername) -1);
    strncpy(tmp.hostname, server.hostname.toUtf8().data(), sizeof(tmp.hostname) -1);
    tmp.port = server.port;
    strncpy(tmp.username, server.username.toUtf8().data(), sizeof(tmp.username) -1);
    strncpy(tmp.password, server.password.toUtf8().data(), sizeof(tmp.password) -1);
    tmp.max_threads = server.max_threads;
    tmp.priority = (NGServerPriority) server.priority;
    tmp.use_ssl = server.use_ssl;
    tmp.enabled = server.enabled;

    bool ret = nntpgrab_glue_config_edit_server(d->glue, servername.toUtf8(), tmp, &errmsg_tmp);

    if (!ret) {
        errmsg.clear();
        errmsg.append(errmsg_tmp);
        ng_free(errmsg_tmp);
    }

    return ret;
}

QNNTPGrabGlue::ConfigOpts QNNTPGrabGlue::configGetOpts()
{
    Q_D(QNNTPGrabGlue);
    NGConfigOpts opts = nntpgrab_glue_config_get_opts(d->glue);
    QNNTPGrabGlue::ConfigOpts ret;

    ret.download_directory = QString(opts.download_directory);
    ret.temp_directory = QString(opts.temp_directory);
    ret.enable_intelligent_par2_downloading = opts.enable_intelligent_par2_downloading;
    ret.enable_par2_repair = opts.enable_par2_repair;
    ret.auto_import_directory = QString(opts.auto_import_directory);
    ret.enable_auto_import = opts.enable_auto_import;
    ret.move_file_after_auto_import = opts.move_file_after_auto_import;
    ret.enable_auto_unpack = opts.enable_auto_unpack;
    ret.enable_bandwidth_shaping = opts.enable_bandwidth_shaping;
    ret.max_bandwidth = opts.max_bandwidth;
    ret.enable_webserver = opts.enable_webserver;
    ret.webserver_port = opts.webserver_port;
    ret.enable_logger = opts.enable_logger;
    ret.auto_remove_files = opts.auto_remove_files;

    return ret;
}

void QNNTPGrabGlue::configSetOpts(QNNTPGrabGlue::ConfigOpts opts)
{
    Q_D(QNNTPGrabGlue);
    NGConfigOpts tmp;
    memset(&tmp, 0, sizeof(tmp));
    strncpy(tmp.download_directory, opts.download_directory.toUtf8().data(), sizeof(tmp.download_directory) - 1);
    strncpy(tmp.temp_directory, opts.temp_directory.toUtf8().data(), sizeof(tmp.temp_directory) - 1);
    tmp.enable_intelligent_par2_downloading = opts.enable_intelligent_par2_downloading;
    tmp.enable_par2_repair = opts.enable_par2_repair;
    strncpy(tmp.auto_import_directory, opts.auto_import_directory.toUtf8().data(), sizeof(tmp.auto_import_directory) - 1);
    tmp.enable_auto_import = opts.enable_auto_import;
    tmp.move_file_after_auto_import = opts.move_file_after_auto_import;
    tmp.enable_auto_unpack = opts.enable_auto_unpack;
    tmp.enable_bandwidth_shaping = opts.enable_bandwidth_shaping;
    tmp.max_bandwidth = opts.max_bandwidth;
    tmp.enable_webserver = opts.enable_webserver;
    tmp.webserver_port = opts.webserver_port;
    tmp.enable_logger = opts.enable_logger;
    tmp.auto_remove_files = opts.auto_remove_files;

    nntpgrab_glue_config_set_opts(d->glue, tmp);
}

bool QNNTPGrabGlue::configGetFolderListing(QString parent, QList<QNNTPGrabGlue::Folder> folders)
{
    Q_D(QNNTPGrabGlue);
    NGList *list_folders, *list;

    if (!nntpgrab_glue_config_get_folder_listing(d->glue, parent.toUtf8(), &list_folders)) {
        return false;
    }

    list = list_folders;
    while (list) {
        NNTPGrabFolder *item = (NNTPGrabFolder*) list->data;
        QNNTPGrabGlue::Folder new_item;

        new_item.folder = item->folder;
        new_item.has_subfolders = item->has_subfolders;

        folders.append(new_item);

        list = ng_list_next(list);
    }
    nntpgrab_glue_config_free_folder_listing(d->glue, list_folders);

    return true;
}

bool QNNTPGrabGlue::startSchedular()
{
    Q_D(QNNTPGrabGlue);
    return nntpgrab_glue_schedular_start(d->glue);
}

bool QNNTPGrabGlue::stopSchedular(bool wait)
{
    Q_D(QNNTPGrabGlue);
    return nntpgrab_glue_schedular_stop(d->glue, wait);
}

QNNTPGrabGlue::SchedularState QNNTPGrabGlue::getSchedularState()
{
    Q_D(QNNTPGrabGlue);
    return (QNNTPGrabGlue::SchedularState) nntpgrab_glue_schedular_get_state(d->glue);
}

bool QNNTPGrabGlue::schedularAddTaskToQueue(QString collection_name, QString subject, QString poster, QDateTime stamp, quint64 file_size, QList<QString> groups, QList<QNNTPGrabGlue::Part> parts, QString &errmsg)
{
    Q_D(QNNTPGrabGlue);
    NGList *list_groups = NULL;
    NGList *list_parts = NULL;
    NGList *list;
    bool ret;
    int i;
    char *errmsg_tmp = NULL;

    for (i = 0; i < groups.length(); i++) {
        list_groups = ng_list_append(list_groups, (char*) strdup(groups[i].toUtf8().data()));
    }
    for (i = 0; i < parts.length(); i++) {
        QNNTPGrabGlue::Part part = parts.at(i);
        NNTPGrabPart *part_tmp = (NNTPGrabPart*) calloc(sizeof(NNTPGrabPart), 1);

        strncpy(part_tmp->message_id, part.message_id.toUtf8().data(), sizeof(part_tmp->message_id) - 1);
        part_tmp->part_num = part.part_num;
        part_tmp->size = part.size;

        list_parts = ng_list_append(list_parts, part_tmp);
    }
    ret = nntpgrab_glue_schedular_add_task_to_queue(d->glue, collection_name.toUtf8(), subject.toUtf8(), poster.toUtf8(), stamp.toTime_t(), file_size, list_groups, list_parts, &errmsg_tmp);

    list = list_groups;
    while (list) {
        free(list->data);
        list = ng_list_next(list);
    }

    list = list_parts;
    while (list) {
        free(list->data);
        list = ng_list_next(list);
    }

    ng_list_free(list_groups);
    ng_list_free(list_parts);

    if (!ret) {
        errmsg.clear();
        errmsg.append(errmsg_tmp);
        ng_free(errmsg_tmp);
    }
    return ret;
}

bool QNNTPGrabGlue::schedularDelTaskFromQueue(QString collection_name, QString subject, QString &errmsg)
{
    Q_D(QNNTPGrabGlue);
    char *errmsg_tmp = NULL;
    bool ret = nntpgrab_glue_schedular_del_task_from_queue(d->glue, collection_name.toUtf8(), (subject.isEmpty() ? NULL : subject.toUtf8().data()), &errmsg_tmp);

    if (!ret) {
        errmsg.clear();
        errmsg.append(errmsg_tmp);
        ng_free(errmsg_tmp);
    }

    return ret;
}

bool QNNTPGrabGlue::schedularRestartTask(QString collection_name, QString subject, QString &errmsg)
{
    Q_D(QNNTPGrabGlue);
    char *errmsg_tmp = NULL;
    bool ret = nntpgrab_glue_schedular_restart_task(d->glue, collection_name.toUtf8(), (subject.isEmpty() ? NULL : subject.toUtf8().data()), &errmsg_tmp);

    if (!ret) {
        errmsg.clear();
        errmsg.append(errmsg_tmp);
        ng_free(errmsg_tmp);
    }

    return ret;
}

bool QNNTPGrabGlue::schedularMarkTaskOptional(QString collection_name, QString subject, bool is_optional)
{
    Q_D(QNNTPGrabGlue);
    return nntpgrab_glue_schedular_mark_task_optional(d->glue, collection_name.toUtf8(), subject.toUtf8(), is_optional);
}

bool QNNTPGrabGlue::schedularSaveQueue(QString &errmsg)
{
    Q_D(QNNTPGrabGlue);
    char *errmsg_tmp = NULL;
    bool ret = nntpgrab_glue_schedular_save_queue(d->glue, &errmsg_tmp);

    if (!ret) {
        errmsg.clear();
        errmsg.append(errmsg_tmp);
        ng_free(errmsg_tmp);
    }

    return ret;
}

struct _foreach_cb_data {
    QList<QNNTPGrabGlue::Collection> collections;
    QNNTPGrabGlue::Collection *active_collection;
    QNNTPGrabGlue::File *active_file;
};

static void foreach_collection_func(const char *collection_name, const char *poster, nguint64 total_size, nguint64 total_size_remaining, int position, void *data)
{
    struct _foreach_cb_data *cb_data = (struct _foreach_cb_data*) data;
    QNNTPGrabGlue::Collection collection;

    collection.collection_name = QString(collection_name);
    collection.poster = QString(poster);
    collection.total_size = total_size;
    collection.total_size_remaining = total_size_remaining;
    collection.position = position;

    cb_data->collections.append(collection);
    cb_data->active_collection = &cb_data->collections[cb_data->collections.count() - 1];
    cb_data->active_file = NULL;
}

static void foreach_file_func(const char *collection_name, const char *subject, const char *poster, ngint64 stamp, nguint64 file_size, nguint64 file_size_remaining, int position, int num_parts_total, int num_parts_downloaded, int num_parts_failed, NGTaskState status, const char *filename, void *data)
{
    struct _foreach_cb_data *cb_data = (struct _foreach_cb_data*) data;
    QNNTPGrabGlue::File file;

    file.collection_name = QString(collection_name);
    file.subject = QString(subject);
    file.poster = QString(poster);
    file.stamp = QDateTime::fromTime_t(stamp);
    file.file_size = file_size;
    file.file_size_remaining = file_size_remaining;
    file.position = position;
    file.num_parts_total = num_parts_total;
    file.num_parts_downloaded = num_parts_downloaded;
    file.num_parts_failed = num_parts_failed;
    file.status = (QNNTPGrabGlue::TaskState) status;
    file.filename = QString(filename);

    Q_ASSERT(cb_data->active_collection != NULL);
    Q_ASSERT(cb_data->active_collection->collection_name == QString(collection_name));

    cb_data->active_collection->files.append(file);
    cb_data->active_file = &cb_data->active_collection->files[cb_data->active_collection->files.count() - 1];
}

static void foreach_group_func (const char *collection_name, const char *subject, const char *group, void *data)
{
    struct _foreach_cb_data *cb_data = (struct _foreach_cb_data*) data;

    Q_ASSERT(cb_data->active_file != NULL);
    Q_ASSERT(cb_data->active_file->collection_name == QString(collection_name));
    Q_ASSERT(cb_data->active_file->subject == QString(subject));

    cb_data->active_file->groups.append(QString(group));
}

QList<QNNTPGrabGlue::Collection> QNNTPGrabGlue::schedularGetAllTasks(void)
{
    Q_D(QNNTPGrabGlue);
    struct _foreach_cb_data cb_data;

    nntpgrab_glue_schedular_foreach_task(d->glue, foreach_collection_func, foreach_file_func, foreach_group_func, &cb_data);

    return cb_data.collections;
}

bool QNNTPGrabGlue::schedularMoveTask(QString collection_name_src, QString subject_src, QString collection_name_dest, int position_dest)
{
    Q_D(QNNTPGrabGlue);
    return nntpgrab_glue_schedular_move_task(d->glue, collection_name_src.toUtf8(), subject_src.toUtf8(), collection_name_dest.toUtf8(), position_dest);
}

bool QNNTPGrabGlue::schedularMoveCollection(QString collection_name, int new_position)
{
    Q_D(QNNTPGrabGlue);
    return nntpgrab_glue_schedular_move_collection(d->glue, collection_name.toUtf8(), new_position);
}

QList<QString> QNNTPGrabGlue::getAvailPlugins()
{
    Q_D(QNNTPGrabGlue);
    QList<QString> ret;
    NGList *plugins, *list;

    plugins = nntpgrab_glue_plugins_get_avail_plugins(d->glue);
    list = plugins;
    while (list) {
        const char *plugin_name = (const char *) list->data;

        ret.append(QString(plugin_name));

        list = ng_list_next(list);
    }
    nntpgrab_glue_plugins_free_avail_plugins(d->glue, plugins);

    return ret;
}

bool QNNTPGrabGlue::getPluginInfo(QString plugin_name, QNNTPGrabGlue::PluginInfo &plugin_info)
{
    Q_D(QNNTPGrabGlue);
    NNTPGrabPluginInfo tmp;
    memset(&tmp, 0, sizeof(tmp));

    bool ret = nntpgrab_glue_plugins_get_plugin_info(d->glue, plugin_name.toUtf8(), &tmp);
    if (ret) {
        plugin_info.name = QString(tmp.name);
        plugin_info.version = QString(tmp.version);
        plugin_info.author = QString(tmp.author);
        plugin_info.url = QString(tmp.url);
        plugin_info.description = QString(tmp.description);
        plugin_info.is_loaded = tmp.is_loaded;
        plugin_info.is_persistent = tmp.is_persistent;
    }

    return ret;
}

bool QNNTPGrabGlue::loadPlugin(QString plugin_name, QString &errmsg)
{
    Q_D(QNNTPGrabGlue);
    bool ret;
    char *errmsg_tmp = NULL;

    ret = nntpgrab_glue_plugins_load_plugin(d->glue, plugin_name.toUtf8(), &errmsg_tmp);
    if (!ret) {
        errmsg.clear();
        errmsg.append(errmsg_tmp);
        ng_free(errmsg_tmp);
    }

    return ret;
}

bool QNNTPGrabGlue::unloadPlugin(QString plugin_name, QString &errmsg)
{
    Q_D(QNNTPGrabGlue);
    bool ret;
    char *errmsg_tmp = NULL;

    ret = nntpgrab_glue_plugins_unload_plugin(d->glue, plugin_name.toUtf8(), &errmsg_tmp);
    if (!ret) {
        errmsg.clear();
        errmsg.append(errmsg_tmp);
        ng_free(errmsg_tmp);
    }

    return ret;
}

bool QNNTPGrabGlue::setPluginPersistent(QString plugin_name, bool persistent)
{
    Q_D(QNNTPGrabGlue);
    return nntpgrab_glue_plugins_set_persistent(d->glue, plugin_name.toUtf8(), persistent);
}

void QNNTPGrabGlue::setEmitLogMessages(bool val)
{
    Q_D(QNNTPGrabGlue);
    nntpgrab_glue_set_emit_log_messages(d->glue, val);
}

QString QNNTPGrabGlue::utilsCalculateFileSize(quint64 file_size)
{
    char buf[16];
    memset(&buf, 0, sizeof(buf));

    nntpgrab_utils_calculate_file_size(file_size, (char*) &buf, sizeof(buf));

    return QString(buf);
}

int QNNTPGrabGlue::utilsCalculateEstimatedTimeRemaining(int bytes_received1, int bytes_received2, int bytes_received3, int bytes_received4, int bytes_received5, int bytes_received6, int bytes_received7, int bytes_received8, int bytes_received9, int bytes_received10, quint64 file_size)
{
    return nntpgrab_utils_calculate_estimated_time_remaining(bytes_received1, bytes_received2, bytes_received3, bytes_received4,
                                                             bytes_received5, bytes_received6, bytes_received7, bytes_received8,
                                                             bytes_received9, bytes_received10, file_size);
}

QString QNNTPGrabGlue::utilsGetReadableTimeRemaining(int estimated_time_remaining)
{
    char buf[64];
    memset(&buf, 0, sizeof(buf));

    nntpgrab_utils_get_readable_time_remaining(estimated_time_remaining, (char*) &buf, sizeof(buf));

    return QString(buf);
}

QString QNNTPGrabGlue::utilsGetReadableFinishedTime(int estimated_time_remaining)
{
    char buf[64];
    memset(&buf, 0, sizeof(buf));

    nntpgrab_utils_get_readable_finished_time(estimated_time_remaining, (char*) &buf, sizeof(buf));

    return QString(buf);
}

bool QNNTPGrabGlue::utilsParseNZBFile(QString nzb_data, QNNTPGrabGlue::NZB &nzbfile, QString &errmsg)
{
    char *errmsg_tmp = NULL;
    NNTPGrabNZB *nzbfile_tmp = nntpgrab_utils_parse_nzb_file(nzb_data.toUtf8().data(), &errmsg_tmp);
    NGList *list;

    if (!nzbfile_tmp) {
        errmsg.clear();
        errmsg.append(errmsg_tmp);
        ng_free(errmsg_tmp);
        return FALSE;
    }

    list = nzbfile_tmp->files;
    while (list) {
        NNTPGrabNZBFile *file_tmp = (NNTPGrabNZBFile*) list->data;

        QNNTPGrabGlue::NZBFile file;

        file.subject = file_tmp->subject;
        file.poster = file_tmp->poster;
        file.stamp = QDateTime::fromTime_t(file_tmp->stamp);
        file.file_size = file_tmp->filesize;

        NGList *list2;

        list2 = file_tmp->segments;
        while (list2) {
            NNTPGrabPart *part_tmp = (NNTPGrabPart*) list2->data;

            QNNTPGrabGlue::Part part;

            part.message_id = part_tmp->message_id;
            part.part_num = part_tmp->part_num;
            part.size = part_tmp->size;

            file.segments.append(part);

            list2 = ng_list_next(list2);
        }

        list2 = file_tmp->groups;
        while (list2) {
            const char *groupname = (const char*) list2->data;

            file.groups.append(QString(groupname));

            list2 = ng_list_next(list2);
        }

        nzbfile.files.append(file);

        list = ng_list_next(list);
    }

    nntpgrab_utils_nzb_file_free(nzbfile_tmp);

    return TRUE;
}

QString QNNTPGrabGlue::utilsSanitizeCollectionName(QString collection_name)
{
    char collection_name_tmp[128];
    memset(&collection_name_tmp, 0, sizeof(collection_name_tmp));

    strncpy(collection_name_tmp, collection_name.toUtf8().data(), sizeof(collection_name_tmp) - 1);

    nntpgrab_utils_sanitize_collection_name(collection_name_tmp);

    return QString(collection_name_tmp);
}

bool QNNTPGrabGlue::utilsTestIsServerAlreadyRunning(void)
{
    return nntpgrab_utils_test_is_server_already_running();
}

bool QNNTPGrabGlue::utilsPerformShutdown(void)
{
    return nntpgrab_utils_perform_shutdown();
}
