#ifndef QNNTPGRABGLUE_P_H
#define QNNTPGRABGLUE_P_H

#include <QThread>
#include "QNNTPGrabGlue.h"
#include "nntpgrab_glue.h"

class QNNTPGrabGlueThread : public QThread
{
    Q_OBJECT

public:
    QNNTPGrabGlueThread (NntpgrabGlue *glue);
    ~QNNTPGrabGlueThread();

private:
    NntpgrabGlue *glue;

protected:
    void run();
};

class QNNTPGrabGluePrivate
{
    Q_DECLARE_PUBLIC(QNNTPGrabGlue)

public:
    QNNTPGrabGluePrivate(QNNTPGrabGlue *parent);
    ~QNNTPGrabGluePrivate();

    NntpgrabGlue *glue;
    QNNTPGrabGlue *q_ptr;

private:
    QNNTPGrabGlueThread *callbacks_thread;

    static void connectionLostWrapper(NntpgrabGlue *glue, const char *errmsg, void *data);
    static void configChangedWrapper(NntpgrabGlue *glue, void *data);
    static void allDownloadsCompletedWrapper(NntpgrabGlue *glue, void *data);
    static void partDownloadStartWrapper(NntpgrabGlue *glue, const char *servername, int conn_id, const char *collection_name, const char *subject, int part_num, void *data);
    static void partDoneWrapper(NntpgrabGlue *glue, const char *servername, int conn_id, const char *collection_name, const char *subject, int part_num, int size, void *data);
    static void partFailedWrapper(NntpgrabGlue *glue, const char *servername, int conn_id, const char *collection_name, const char *subject, int part_num,int size, bool all_servers_tried, void *data);
    static void trafficMonitorUpdateWrapper(NntpgrabGlue *glue, int bytes_received1, int bytes_received2, int bytes_received3, int bytes_received4, int bytes_received5, int bytes_received6, int bytes_received7, int bytes_received8, int bytes_received9, int bytes_received10, nguint64 stamp, double average, void *data);
    static void partProgressUpdateWrapper(NntpgrabGlue *glue, const char *servername, int conn_id, const char *collection_name, const char *subject, int part_num, int bytes_downloaded, int bytes_total, void *data);
    static void collectionAddedWrapper(NntpgrabGlue *glue, const char *collection_name, const char *poster, void *data);
    static void collectionModifiedWrapper(NntpgrabGlue *glue, const char *collection_name, const char *poster, void *data);
    static void collectionRemovedWrapper(NntpgrabGlue *glue, const char *collection_name, void *data);
    static void fileAddedWrapper(NntpgrabGlue *glue, const char *collection_name, const char *subject, const char *poster, nguint64 stamp, nguint64 file_size, nguint64 total_size, nguint64 total_size_remaining, NGTaskState state, int num_parts, NGList *groups, void *data);
    static void fileRemovedWrapper(NntpgrabGlue *glue, const char *collection_name, const char *subject, nguint64 total_size, nguint64 total_size_remaining, void *data);
    static void fileDownloadStateUpdateWrapper(NntpgrabGlue *glue, const char *collection_name, const char *subject, int num_parts_total, int num_parts_done, int num_parts_failed, nguint64 file_size, nguint64 file_size_remaining, nguint64 total_size, nguint64 total_size_remaining, void *data);
    static void fileStateChangedWrapper(NntpgrabGlue *glue, const char *collection_name, const char *subject, const char *real_filename, NGTaskState old_state, NGTaskState new_state, void *data);
    static void connectionConnectingWrapper(NntpgrabGlue *glue, const char *servername, int conn_id, void *data);
    static void connectionConnectedWrapper(NntpgrabGlue *glue, const char *servername, int conn_id, const char *welcome_msg, void *data);
    static void connectionDisconnectWrapper(NntpgrabGlue *glue, const char *servername, int conn_id, NNTPDisconnectType disconnect_type, const char *reason, void *data);
    static void schedularStateChangedWrapper(NntpgrabGlue *glue, NGSchedularState new_state, const char *reason, void *data);
    static void logMessageWrapper(NntpgrabGlue *glue, const char *component, NGLogLevel log_level, const char *msg, void *data);
    static void fileMovedWrapper(NntpgrabGlue *glue, const char *orig_collection_name, const char *subject, const char *new_collection_name, int old_position, int new_position, void *data);
    static void collectionMovedWrapper(NntpgrabGlue *glue, const char *collection_name, int old_position, int new_position, void *data);
    static void collectionDownloadedWrapper(NntpgrabGlue *glue, const char *collection_name, void *data);
    static void pluginLoadedWrapper(NntpgrabGlue *glue, const char *plugin_name, bool is_persistent, void *data);
    static void pluginUnloadedWrapper(NntpgrabGlue *glue, const char *plugin_name, void *data);
    static void pluginEventWrapper(NntpgrabGlue *glue, const char *plugin_name, const char *event_name, const char **values, void *data);
};

#endif // QNNTPGRABGLUE_P_H
