#include "nntpgrab_utils.h"
#include "QNZBCreator.h"

QNZBCreator::QNZBCreator(QObject *parent) :
    QObject(parent)
{
}

bool QNZBCreator::checkApiVersion(void)
{
    return nntpgrab_utils_nzbcreator_check_api_version();
}

bool QNZBCreator::getAllGroups(QString &errmsg, QList<QNZBCreator::Group> &groups)
{
    char *errmsg_tmp = NULL;
    NGList *groups_tmp, *list;
    QNZBCreator::Group group;

    errmsg.clear();
    groups.clear();

    groups_tmp = nntpgrab_utils_nzbcreator_get_all_groups(&errmsg_tmp);
    if (!groups_tmp) {
        errmsg = errmsg_tmp;
        ng_free(errmsg_tmp);
        return false;
    }

    list = groups_tmp;
    while (list) {
        NZBCreatorGroup *group_tmp = (NZBCreatorGroup*) list->data;

        group.group_id = group_tmp->group_id;
        group.newsgroup = group_tmp->newsgroup;
        groups.append(group);

        list = ng_list_next(list);
    }

    nntpgrab_utils_nzbcreator_free_groups(groups_tmp);
    return TRUE;
}

bool QNZBCreator::performSearch(QNZBCreator::SearchOpts opts, QString &errmsg, QNZBCreator::SearchResult &result)
{
    char *errmsg_tmp = NULL;
    NZBCreatorSearchOpts opts_tmp;
    NZBCreatorSearchResult *result_tmp;

    memset(&opts_tmp, 0, sizeof(NZBCreatorSearchResult));
    strncpy(opts_tmp.query, opts.query.toUtf8(), sizeof(opts_tmp.query) - 1);
    strncpy(opts_tmp.username, opts.username.toUtf8(), sizeof(opts_tmp.username) - 1);
    strncpy(opts_tmp.password, opts.password.toUtf8(), sizeof(opts_tmp.password) - 1);
    opts_tmp.search_only_in = (NZBCreatorSearchOnlyIn) opts.search_only_in;
    opts_tmp.max_age = opts.max_age;
    opts_tmp.minimal_file_size = opts.minimal_file_size;
    opts_tmp.maximal_file_size = opts.maximal_file_size;
    opts_tmp.percentage_complete = opts.percentage_complete;
    opts_tmp.group_to_search = opts.group_to_search;
    opts_tmp.file_type = (NNTPFileType) opts.file_type;

    result_tmp = nntpgrab_utils_nzbcreator_perform_search(opts_tmp, &errmsg_tmp);
    if (!result_tmp) {
        errmsg = errmsg_tmp;
        ng_free(errmsg_tmp);
        return false;
    }

    result.number_of_hits = result_tmp->number_of_hits;
    result.number_of_results = result_tmp->number_of_results;

    NGList *list = result_tmp->collections;
    while (list) {
        NZBCreatorCollection *collection_tmp = (NZBCreatorCollection*) list->data;
        QNZBCreator::Collection collection;

        collection.collection_name = collection_tmp->collection_name;
        collection.short_collection_name = collection_tmp->short_collection_name;
        collection.newsgroup = collection_tmp->newsgroup;
        collection.stamp = QDateTime::fromTime_t(collection_tmp->stamp);
        collection.total_file_size = collection_tmp->total_file_size;

        NGList *list2 = collection_tmp->files;
        while (list2) {
            NZBCreatorFile *file_tmp = (NZBCreatorFile*) list2->data;
            QNZBCreator::File file;

            file.file_id = file_tmp->file_id;
            file.subject = file_tmp->subject;
            file.poster = file_tmp->poster;
            file.file_type = (QNZBCreator::FileType) file_tmp->file_type;
            file.file_size = file_tmp->file_size;
            file.complete_percentage = file_tmp->complete_percentage;
            file.stamp = QDateTime::fromTime_t(file_tmp->stamp);
            file.num_parts_expected = file_tmp->num_parts_expected;
            file.num_parts_found = file_tmp->num_parts_found;

            collection.files.append(file);

            list2 = ng_list_next(list2);
        }

        result.collections.append(collection);

        list = ng_list_next(list);
    }

    nntpgrab_utils_nzbcreator_free_result(result_tmp);
    return true;
}

QString QNZBCreator::generateNZB(QList<int> file_ids, QString &errmsg)
{
    NGList *file_ids_tmp = NULL;
    char *errmsg_tmp = NULL;
    char *result_tmp;

    Q_FOREACH(int file_id, file_ids) {
        file_ids_tmp = ng_list_append(file_ids_tmp, (void*) (int)file_id);
    }

    result_tmp = nntpgrab_utils_nzbcreator_generate_NZB(file_ids_tmp, &errmsg_tmp);
    ng_list_free(file_ids_tmp);
    if (!result_tmp) {
        errmsg = errmsg_tmp;
        ng_free(errmsg_tmp);
        return QString();
    } else {
        QString result = result_tmp;
        ng_free(result_tmp);
        return result;
    }
}
