#include "WidgetConfigPAR2.h"
#include "ui_WidgetConfigPAR2.h"

WidgetConfigPAR2::WidgetConfigPAR2(QNNTPGrabGlue *glue, QWidget *parent) :
    QWidget(parent),
    ui(new Ui::WidgetConfigPAR2)
{
    this->glue = glue;

    ui->setupUi(this);

    onConfigChanged();

    connect(ui->chkEnablePAR2Repair, SIGNAL(toggled(bool)), SLOT(chkEnablePAR2Repair_toggled(bool)));
    connect(ui->chkEnableIntelligentPar2Downloading, SIGNAL(toggled(bool)), SLOT(chkEnableIntelligentPAR2Downloading_toggled(bool)));
    connect(ui->chkEnableAutoUnpack, SIGNAL(toggled(bool)), SLOT(chkEnableAutoUnpack_toggled(bool)));
    connect(ui->chkRemoveFilesAfterRepairAndUnpack, SIGNAL(toggled(bool)), SLOT(chkRemoveFilesAfterRepairAndUnpack_toggled(bool)));
    connect(glue, SIGNAL(configChanged(void)), SLOT(onConfigChanged(void)));
}

WidgetConfigPAR2::~WidgetConfigPAR2()
{
    delete ui;
}

void WidgetConfigPAR2::changeEvent(QEvent *e)
{
    QWidget::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void WidgetConfigPAR2::onConfigChanged(void)
{
    QNNTPGrabGlue::ConfigOpts opts = glue->configGetOpts();
    ui->chkEnablePAR2Repair->setChecked(opts.enable_par2_repair);
    ui->chkEnableIntelligentPar2Downloading->setChecked(opts.enable_intelligent_par2_downloading);
    ui->chkEnableAutoUnpack->setChecked(opts.enable_auto_unpack);
    ui->chkRemoveFilesAfterRepairAndUnpack->setChecked(opts.auto_remove_files);
}

void WidgetConfigPAR2::chkEnablePAR2Repair_toggled(bool toggled)
{
    qDebug("chkEnablePAR2Repair_toggled: toggled = %i", toggled);

    QNNTPGrabGlue::ConfigOpts opts = glue->configGetOpts();
    opts.enable_par2_repair = toggled;
    glue->configSetOpts(opts);
}

void WidgetConfigPAR2::chkEnableIntelligentPAR2Downloading_toggled(bool toggled)
{
    qDebug("chkEnableIntelligentPAR2Downloading_toggled: toggled = %i", toggled);

    QNNTPGrabGlue::ConfigOpts opts = glue->configGetOpts();
    opts.enable_intelligent_par2_downloading = toggled;
    glue->configSetOpts(opts);
}

void WidgetConfigPAR2::chkEnableAutoUnpack_toggled(bool toggled)
{
    qDebug("chkEnableAutoUnpack_toggled: toggled = %i", toggled);

    QNNTPGrabGlue::ConfigOpts opts = glue->configGetOpts();
    opts.enable_auto_unpack = toggled;
    glue->configSetOpts(opts);
}

void WidgetConfigPAR2::chkRemoveFilesAfterRepairAndUnpack_toggled(bool toggled)
{
    qDebug("chkRemoveFilesAfterRepairAndUnpack_toggled: toggled = %i", toggled);

    QNNTPGrabGlue::ConfigOpts opts = glue->configGetOpts();
    opts.auto_remove_files = toggled;
    glue->configSetOpts(opts);
}


