#ifndef WIDGETNZBCREATOR_H
#define WIDGETNZBCREATOR_H

#include <QWidget>
#include <QTreeWidgetItem>
#include <QFuture>
#include <QFutureWatcher>
#include "QNZBCreator.h"
#include "QNNTPGrabGlue.h"
#include "ProgressBarDelegate.h"

namespace Ui {
    class WidgetNZBCreator;
}

class WidgetNZBCreator : public QWidget {
    Q_OBJECT
public:
    WidgetNZBCreator(QNNTPGrabGlue *glue, QWidget *parent = 0);
    ~WidgetNZBCreator();

protected:
    void changeEvent(QEvent *e);

private:
    Ui::WidgetNZBCreator *ui;
    QNNTPGrabGlue *glue;
    QNZBCreator nzbcreator;
    ProgressBarDelegate *progressbarDelegate;

    struct SearchResult {
        bool ret;
        QString errmsg;
        QNZBCreator::SearchResult result;
    };

    QFuture<WidgetNZBCreator::SearchResult> future_search;
    QFutureWatcher<WidgetNZBCreator::SearchResult> watcher_search;

    WidgetNZBCreator::SearchResult performSearch(QNZBCreator::SearchOpts opts);

    struct ImportResult {
        QString errmsg;
        QString nzb_data;
    };

    QFuture<WidgetNZBCreator::ImportResult> future_import;
    QFutureWatcher<WidgetNZBCreator::ImportResult> watcher_import;
    QString collection_name;

    WidgetNZBCreator::ImportResult performImport(QList<int> file_ids);

private slots:
    void btnSearch_clicked();
    void btnImport_clicked();
    void btnSelectAll_clicked();
    void btnSelectNone_clicked();
    void entryQuery_textChanged(QString text);
    void treeWidget_itemChanged(QTreeWidgetItem *item, int column);

    void performSearch_finished();
    void performImport_finished();
};

#endif // WIDGETNZBCREATOR_H
