/*
    Copyright (C) 2005-2010  Erik van Pienbroek

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef _MSC_VER
#include "config.h.win32"
#else
#include "config.h"
#endif

#include <stdlib.h>
#include <string.h>

#include <glib.h>
#include <glib-object.h>

#include "nntpgrab_glue.h"
#include "nntpgrab_glue_internal.h"
#include "nntpgrab_utils.h"
#include "nntpgrab_internal.h"
#include "collections.h"

void
nntpgrab_glue_signal_connect(NntpgrabGlue *obj, const char *signal_name, NGCallback cb_handler, void *data)
{
    NntpgrabGlue *glue = NNTPGRAB_GLUE(obj);

    g_return_if_fail(glue != NULL);
    g_return_if_fail(signal_name != NULL);

    if (glue->standalone_core && strcmp(signal_name, "connection_lost")) {
        ng_signal_connect(glue->standalone_core, signal_name, cb_handler, data);
    } else {
        ng_signal_connect(glue, signal_name, cb_handler, data);
    }
}

void
nntpgrab_glue_signal_handlers_disconnect_by_func(NntpgrabGlue *obj, NGCallback cb_handler, void *data)
{
    NntpgrabGlue *glue = NNTPGRAB_GLUE(obj);

    g_return_if_fail(glue != NULL);

    if (glue->standalone_core) {
        ng_signal_handlers_disconnect_by_func(glue->standalone_core, cb_handler, data);
    } else {
        ng_signal_handlers_disconnect_by_func(glue, cb_handler, data);
    }
}

void
nntpgrab_glue_signal_handlers_block_by_func(NntpgrabGlue *obj, NGCallback cb_handler, void *data)
{
    NntpgrabGlue *glue = NNTPGRAB_GLUE(obj);

    g_return_if_fail(glue != NULL);

    if (glue->standalone_core) {
        ng_signal_handlers_block_by_func(glue->standalone_core, cb_handler, data);
    } else {
        ng_signal_handlers_block_by_func(glue, cb_handler, data);
    }
}

void
nntpgrab_glue_signal_handlers_unblock_by_func(NntpgrabGlue *obj, NGCallback cb_handler, void *data)
{
    NntpgrabGlue *glue = NNTPGRAB_GLUE(obj);

    g_return_if_fail(glue != NULL);

    if (glue->standalone_core) {
        ng_signal_handlers_unblock_by_func(glue->standalone_core, cb_handler, data);
    } else {
        ng_signal_handlers_unblock_by_func(glue, cb_handler, data);
    }
}
