/*
    Copyright (C) 2005-2010  Erik van Pienbroek

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _NNTPCONNECTION_H_
#define _NNTPCONNECTION_H_

#include <sys/time.h>

#include "configuration.h"
#include "collections.h"

#ifdef HAVE_SSL
#ifdef HAVE_GNUTLS
#include <gnutls/gnutls.h>
#elif defined(HAVE_NSS_COMPAT_OSSL)
#include <nss_compat_ossl/nss_compat_ossl.h>
#else
#include <openssl/ssl.h>
#include <openssl/err.h>
#endif
#endif

typedef enum {
    NNTP_JOB_TYPE_NONE,
    NNTP_JOB_TYPE_ARTICLE,
    NNTP_JOB_TYPE_XOVER
} NNTPJobType;

typedef struct _nntp_connection_info NNTPConnectionInfo;

struct _nntp_connection_info {
    int connection_id;
    int server_id;
    NGConfigServer server_info;
    GPollFD poll_fd;
#ifdef HAVE_SSL
#ifdef HAVE_GNUTLS
    gnutls_session_t ssl_session;
#else
    SSL *ssl;
#endif
#endif
    gboolean is_idle;
    time_t idle_start_stamp;
    time_t last_checked_stamp;
    time_t last_disconnect_stamp;
    time_t last_activity_stamp;
    char recv_buffer2[65536];
    int recv_buffer_length2;
    char recv_buffer[65536];
    int recv_buffer_length;
    char article_filename[1024];
    int article_fd;
    int article_bytes_downloaded;
    struct timeval last_article_progress_announce;
    char article_write_buffer[128 * 1024];              /* Flush the received article data every 128KB */
    int article_write_buffer_length;
    void (*active_method) (NNTPConnectionInfo *conn);
    int max_bandwidth;

    NNTPJobType job_type;

    NNTPCollection *collection;
    NNTPFile *file;
    NNTPPart *part;

    char newsgroup[256];
    gint64 xover_start_range;
    gint64 xover_end_range;
};

typedef enum {
    NNTP_CONNECTION_ERROR_NONE,
    NNTP_CONNECTION_ERROR_HOST_NOT_FOUND,
    NNTP_CONNECTION_ERROR_SSL_INITIALISE,
    NNTP_CONNECTION_ERROR_CONNECTION_REFUSED,
    NNTP_CONNECTION_ERROR_CONNECTION_TIMEOUT,
    NNTP_CONNECTION_ERROR_READ,
    NNTP_CONNECTION_ERROR_WRITE,
    NNTP_CONNECTION_ERROR_INVALID_MSG,
    NNTP_CONNECTION_ERROR_INVALID_ARGUMENT,
    NNTP_CONNECTION_ERROR_LOGIN_FAILURE,
    NNTP_CONNECTION_ERROR_TOO_MANY_CONNECTIONS,
    NNTP_CONNECTION_ERROR_PART_NOT_AVAILABLE,
    NNTP_CONNECTION_ERROR_NO_SUCH_GROUP,
    NNTP_CONNECTION_ERROR_UNABLE_TO_SAVE_PART,
} NNTPConnectionErrCode;

void                     nntpconnection_init_libproxy(void);
NNTPConnectionErrCode    nntpconnection_connect_to_server(NNTPConnectionInfo *conn, char **errmsg);
void                     nntpconnection_disconnect_from_server(NNTPConnectionInfo *conn, NNTPDisconnectType disconnect_type, const char *reason, const char *cause_file, int cause_lineno);
void                     nntpconnection_send_body_command(NNTPConnectionInfo *conn);
void                     nntpconnection_process_socket_activity(NNTPConnectionInfo *conn);

#endif /* _NNTPCONNECTION_H_ */
