#ifndef NNTPGRAB_QT_H
#define NNTPGRAB_QT_H

#include <QObject>
#include <QString>
#include <QThread>

#include "nntpgrab_types.h"
#include "nntpgrab.h"

class QNNTPGrabThread : public QThread
{
    Q_OBJECT

private:
    NntpgrabCore *core;

public:
    QNNTPGrabThread (NntpgrabCore *core);
    ~QNNTPGrabThread();

protected:
    void run();
};

class QNNTPGrab : public QObject
{
    Q_OBJECT

public:
    QNNTPGrab();
    ~QNNTPGrab();

    enum NNTPDisconnectType {
        DISCONNECT_NORMAL,
        DISCONNECT_NO_SUCH_HOST,
        DISCONNECT_CONNECTION_REFUSED,
        DISCONNECT_TOO_MANY_CONNECTIONS,
        DISCONNECT_CONNECT_TIMEOUT,
        DISCONNECT_READ_ERROR,
        DISCONNECT_READ_TIMEOUT,
        DISCONNECT_WRITE_ERROR,
        DISCONNECT_IDLE_TIMEOUT,
        DISCONNECT_INVALID_MSG,
        DISCONNECT_LOGIN_FAILURE,
        DISCONNECT_ERROR_SSL_INITIALISE,
        DISCONNECT_UNEXPECTED
    };

    enum TaskState {
        TASK_STATE_WAITING_FOR_DOWNLOAD,
        TASK_STATE_DOWNLOADING,
        TASK_STATE_WAITING_FOR_DECODE,
        TASK_STATE_DECODING,
        TASK_STATE_FINISHED_COMPLETE,
        TASK_STATE_FINISHED_INCOMPLETE,
        TASK_STATE_FINISHED_NO_PARTS_AVAIL,
        TASK_STATE_SKIPPED
    };

    enum LogLevel {
        LOG_LEVEL_ALL = 0,
        LOG_LEVEL_INFO,
        LOG_LEVEL_WARNING,
        LOG_LEVEL_ERROR,
        LOG_LEVEL_FATAL,
        LOG_LEVEL_DEBUG,
    };

    enum SchedularState {
        SCHEDULAR_STATE_RUNNING,
        SCHEDULAR_STATE_STOPPING,
        SCHEDULAR_STATE_STOPPED
    };

    bool init(char **errmsg, char **warnings);
    NGConfigOpts configGetOpts();
    SchedularState getSchedularState();
    bool startSchedular();
    bool stopSchedular(bool wait);
    bool loadPlugin(QString plugin_name, QString *errmsg);
    bool startEmbeddedServer(int port, char **errmsg);

private:
    NntpgrabCore *core;
    QNNTPGrabThread *callbacks_thread;

    static void configChangedWrapper(NntpgrabCore *core, void *data);
    static void allDownloadsCompletedWrapper(NntpgrabCore *core, void *data);
    static void partDownloadStartWrapper(NntpgrabCore *core, const char *servername, int conn_id, const char *collection_name, const char *subject, int part_num, void *data);
    static void partDoneWrapper(NntpgrabCore *core, const char *servername, int conn_id, const char *collection_name, const char *subject, int part_num, int size, void *data);
    static void partFailedWrapper(NntpgrabCore *core, const char *servername, int conn_id, const char *collection_name, const char *subject, int part_num,int size, bool all_servers_tried, void *data);
    static void trafficMonitorUpdateWrapper(NntpgrabCore *core, int bytes_received1, int bytes_received2, int bytes_received3, int bytes_received4, int bytes_received5, int bytes_received6, int bytes_received7, int bytes_received8, int bytes_received9, int bytes_received10, nguint64 stamp, double average, void *data);
    static void partProgressUpdateWrapper(NntpgrabCore *core, const char *servername, int conn_id, const char *collection_name, const char *subject, int part_num, int bytes_downloaded, int bytes_total, void *data);
    static void collectionAddedWrapper(NntpgrabCore *core, const char *collection_name, const char *poster, void *data);
    static void collectionModifiedWrapper(NntpgrabCore *core, const char *collection_name, const char *poster, void *data);
    static void collectionRemovedWrapper(NntpgrabCore *core, const char *collection_name, void *data);
    static void fileAddedWrapper(NntpgrabCore *core, const char *collection_name, const char *subject, const char *poster, nguint64 stamp, nguint64 file_size, nguint64 total_size, nguint64 total_size_remaining, TaskState state, int num_parts, NGList *groups, void *data);
    static void fileRemovedWrapper(NntpgrabCore *core, const char *collection_name, const char *subject, nguint64 total_size, nguint64 total_size_remaining, void *data);
    static void fileDownloadStateUpdateWrapper(NntpgrabCore *core, const char *collection_name, const char *subject, int num_parts_total, int num_parts_done, int num_parts_failed, nguint64 file_size, nguint64 file_size_remaining, nguint64 total_size, nguint64 total_size_remaining, void *data);
    static void fileStateChangedWrapper(NntpgrabCore *core, const char *collection_name, const char *subject, const char *real_filename, TaskState old_state, TaskState new_state, void *data);
    static void connectionConnectingWrapper(NntpgrabCore *core, const char *servername, int conn_id, void *data);
    static void connectionConnectedWrapper(NntpgrabCore *core, const char *servername, int conn_id, const char *welcome_msg, void *data);
    static void connectionDisconnectWrapper(NntpgrabCore *core, const char *servername, int conn_id, NNTPDisconnectType disconnect_type, const char *reason, void *data);
    static void schedularStateChangedWrapper(NntpgrabCore *core, NGSchedularState new_state, const char *reason, void *data);
    static void logMessageWrapper(NntpgrabCore *core, const char *component, NGLogLevel log_level, const char *msg, void *data);
    static void fileMovedWrapper(NntpgrabCore *core, const char *orig_collection_name, const char *subject, const char *new_collection_name, int old_position, int new_position, void *data);
    static void collectionMovedWrapper(NntpgrabCore *core, const char *collection_name, int old_position, int new_position, void *data);
    static void collectionDownloadedWrapper(NntpgrabCore *core, const char *collection_name, void *data);
    static void pluginLoadedWrapper(NntpgrabCore *core, const char *plugin_name, bool is_persistent, void *data);
    static void pluginUnloadedWrapper(NntpgrabCore *core, const char *plugin_name, void *data);
    static void pluginEventWrapper(NntpgrabCore *core, const char *plugin_name, const char *event_name, const char **values, void *data);

signals:
    void configChanged();
    void allDownloadsCompleted();
    void partDownloadStart(QString servername, int conn_id, QString collection_name, QString subject, int part_num);
    void partDone(QString servername, int conn_id, QString collection_name, QString subject, int part_num, int size);
    void partFailed(QString servername, int conn_id, QString collection_name, QString subject, int part_num, int size, bool all_servers_tried);
    void trafficMonitorUpdate(int bytes_received[10], time_t stamp, double average);
    void partProgressUpdate(QString servername, int conn_id, QString collection_name, QString subject, int part_num, int bytes_downloaded, int bytes_total);
    void collectionAdded(QString collection_name, QString poster);
    void collectionModified(QString collection_name, QString poster);
    void collectionRemoved(QString collection_name);
    void fileAdded(QString collection_name, QString subject, QString poster, time_t stamp, quint64 file_size, quint64 total_size, quint64 total_size_remaining, QNNTPGrab::TaskState state, int num_parts, NGList *groups);
    void fileRemoved(QString collection_name, QString subject, quint64 total_size, quint64 total_size_remaining);
    void fileDownloadStateUpdate(QString collection_name, QString subject, int num_parts_total, int num_parts_done, int num_parts_failed, quint64 file_size, quint64 file_size_remaining, quint64 total_size, quint64 total_size_remaining);
    void fileStateChanged(QString collection_name, QString subject, QString real_filename, QNNTPGrab::TaskState old_state, QNNTPGrab::TaskState new_state);
    void connectionConnecting(QString servername, int conn_id);
    void connectionConnected(QString servername, int conn_id, QString welcome_msg);
    void connectionDisconnect(QString servername, int conn_id, QNNTPGrab::NNTPDisconnectType disconnect_type, QString reason);
    void schedularStateChanged(QNNTPGrab::SchedularState new_state, QString reason);
    void logMessage(QString component, QNNTPGrab::LogLevel log_level, QString msg);
    void fileMoved(QString orig_collection_name, QString subject, QString new_collection_name, int old_position, int new_position);
    void collectionMoved(QString collection_name, int old_position, int new_position);
    void collectionDownloaded(QString collection_name);
    void pluginLoaded(QString plugin_name, bool is_persistent);
    void pluginUnloaded(QString plugin_name);
    void pluginEvent(QString plugin_name, QString event_name, QList<QString> values);
};

#endif // NNTPGRAB_QT_H
